function [lambda,lsqobjfnval,exitflag,output] =... 
    FilterOpt(obsdata,ProbNgNtilde,lambda0,output,solver,Adjustment)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
%%%%%%%%%%%%  IMPORTANT NOTE:  the notation in the code here regarding N vs
%%%%%%%%%%%%  Ntilde is the reverse of the new version of the paper as of
%%%%%%%%%%%%  September 2015!  The paper denotes observed bidders as N
%%%%%%%%%%%%  and total potential bidders (but unobserved) as Ntilde.  HERE
%%%%%%%%%%%%  IN THE CODE, THE REVERSE IS TRUE, reflecting a notation
%%%%%%%%%%%%  convention which pre-dates the current revision of the paper:
%%%%%%%%%%%%  in this program Ntilde is the (unobserved) total
%%%%%%%%%%%%  participation and N is the observed number of bidders.
diagnosticplot = 0;
if nargin<6
    Adjustment = 1;
end
Nsamp = obsdata(:,1);
if Adjustment==1
    fpt   = ~isnan(obsdata(:,3))+0;  %%%fpt stands for first-price trigger.
    L     = length(Nsamp);
    Ndom = unique(Nsamp);
    Pk = zeros(size(Ndom));
    for k=1:length(Pk)
        Pk(k) = sum(fpt==1 & Nsamp==Ndom(k))/sum(Nsamp==Ndom(k));
    end

    display('')
    display('%%%%%%%%%% NOW COMPUTING SIA ADJUSTMENT  %%%%%%%%%%%%%%%')
    display('')
    S = 10000000;
    NsampAdj = nan(S,1);
    parfor s=1:S
        Ndraw = Nsamp(ceil(rand(1)*L)); 
        indx = find(Ndraw==Ndom);
        cointoss = rand(1);
        if cointoss<=Pk(indx) 
            NsampAdj(s) = Ndraw+1;
        else
            NsampAdj(s) = Ndraw;
        end
	end
    if diagnosticplot==1
        figure(7777777)
        hold on
        [F,X] = ecdf(Nsamp);
        stairs(X,F,'--b','LineWidth',3)
        clear F X
    end
    [Femp,sampdom] = ecdf(NsampAdj);  %%%%This is the sample CDF
    clear NsampAdj
    if diagnosticplot==1
        stairs(sampdom,Femp,'r','LineWidth',3)
        legend('RAW $\tilde{N}$ ECDF','ADJUSTED $\tilde{N}$ ECDF','Location','Southeast')
    end
else 
    [Femp,sampdom] = ecdf(Nsamp);  %%%%This is the sample CDF
end

rhohat = Femp(2:end) - Femp(1:end-1);  %%%%This is the sample PDF that will enter the least squares objective function
Femp(1)    = [];
sampdom(1) = [];
if nargin<5
    solver = 'knitro';
end
A    = [];  
b    = [];  
Aeq  = [];  
beq  = [];
lb   = [2; 0];
ub   = [100; 0.99];
nonlcon = [];


xtolerance   = 1e-8;
funtolerance = 1e-8;
constrainttolerance = 1e-8;
constraintgradient = 'off';
disp = 'off';

if output==0 | strcmpi(output,'off')
    if strcmpi(solver,'knitro')
        opts = optimoptions('ktrlink','GradObj',constraintgradient,...
                            'tolx',xtolerance,'tolfun',funtolerance,...
                            'tolcon',constrainttolerance);
    elseif strcmpi(solver,'fmincon')
        opts = optimoptions('fmincon','UseParallel','always',...
                            'Algorithm','active-set','GradObj',constraintgradient,...
                            'tolx',xtolerance,'tolfun',funtolerance,...
                            'tolcon',constrainttolerance);
    end
else
    if strcmpi(solver,'knitro')
        opts = optimoptions('ktrlink','Display',disp,...
                            'GradObj',constraintgradient,'tolx',xtolerance,...
                            'tolfun',funtolerance,'tolcon',constrainttolerance);
    elseif strcmpi(solver,'fmincon')
        opts = optimoptions('fmincon','UseParallel','always',...
                            'Algorithm','active-set','Display',disp,...
                            'GradObj',constraintgradient,'tolx',xtolerance,...
                            'tolfun',funtolerance,'tolcon',constrainttolerance);
    end
end

if strcmpi(solver,'knitro')
    [lambda,lsqobjfnval] = ktrlink(@nestedfun,lambda0,A,b,Aeq,beq,lb,ub,nonlcon,opts);
elseif strcmpi(solver,'fmincon')
    [lambda,lsqobjfnval,exitflag,output] = fmincon(@nestedfun,lambda0,A,b,Aeq,beq,lb,ub,nonlcon,opts);
end


    function [lsqobjfn,grad] = nestedfun(lambda)
        Ntilde         = (1:100)';  %%%%This vector contains hypothetical values of the actual number of bidders (as opposed to the number of observed bidders) which will be used to construct a model-generated PMF of the observed number of bidders.
        P              = poissgenpdf(Ntilde,lambda);  %%%%This is the truncated poisson probabilities (where truncation comes from the fact that we only observe at least one bidder.
        Pprime         = zeros(length(P),2);
        Pprime(:,1)    = P.*( 1/lambda(1) + (Ntilde-1)./(lambda(1)+Ntilde*lambda(2)) - 1 );  %%%This is the derivative of the Poisson probabilities with respect to lambda(1)
        Pprime(:,2)    = P.*( Ntilde.*(Ntilde-1)./(lambda(1)+Ntilde*lambda(2)) - Ntilde );  %%%This is the derivative of the Poisson probabilities with respect to lambda(2)
        rhomodel       = zeros(size(rhohat));  %%%%This vector will contain the model-generated probabilities corresponding to different values of (observed) N in the data
        rhomodelprime  = zeros(length(rhohat),2);  %%%%This vector will contain the model-generated probabilities corresponding to different values of (observed) N in the data
        for i=1:length(rhohat)
            N = sampdom(i);
            rhomodel(i)      = ProbNgNtilde(Ntilde,N)'*P;
            rhomodelprime(i,1) = ProbNgNtilde(Ntilde,N)'*Pprime(:,1);
            rhomodelprime(i,2) = ProbNgNtilde(Ntilde,N)'*Pprime(:,2);
        end
        lsqobjfn  = sum((rhohat-rhomodel).^2);
        grad(1) = -2*sum((rhohat-rhomodel).*rhomodelprime(:,1));
        grad(2) = -2*sum((rhohat-rhomodel).*rhomodelprime(:,2));
    end



end

