function [betaV,betaB,epsilon0,epsilon1,epsilon2,epsilon3,objfunval,exitflag,output] =... 
    VDistSplineEstim(bids,lambda,delta1,delta2,bstar,knotvecV,knotvecB,...
                               vdom,bdom,Fhat,L,solver,beta0,penalty)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%

if nargin<10
    solver = 'fmincon';
end
bids = sort(bids);
dim = size(bids);
if dim(2)~=1
    error('FinalVDistSplineEstim:The bid data vector must be a single column.')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% COMPUTE B-SPLINE BASIS FUNs FOR DERIVATIVE CHECK POINTS:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[BasisB,~,DerivB]           = BsplineBasis3(knotvecB,bids,'off');
[BasisV,~,DerivCheckV]      = BsplineBasis3(knotvecV,vdom,'off');
[BasisCheckB,~,DerivCheckB] = BsplineBasis3(knotvecB,bdom,'off');


display('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
display('%%%%%%%%%% COMPUTE INITIAL GUESS FOR FV:')
display('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')

J   = length(knotvecV)-1;
InitialGuessMethod = 2;
if nargin<13 || isempty(beta0)
    if InitialGuessMethod==1
        tempF    = Fhat.^(1/5);
        tempF    = (tempF-tempF(1))/(1-tempF(1));
        tempBasis = BsplineBasis3(knotvecV,bids,'off');
        tempA = [];
        tempb = [];
        for i=1:J+2  %%%%This gives us monotonicity of the CDF function
            tempA = [tempA; zeros(1,i-1),1,-1,zeros(1,(J+3)-2-(i-1))];
            tempb = [tempb;0];
        end
        tempAeq  = [1,zeros(1,J+2); zeros(1,J+2),1];  %%%%This gives us the constraint that the estimate must pass through 0 at the lower end
        tempbeq  = [0;1];
        lb       = [];
        ub       = [];
        betaV0 = lsqlin(tempBasis,tempF,tempA,tempb,tempAeq,tempbeq,lb,ub);  %%%%This is the initial guess for FV parameters
        
		display('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
        display('%%%%%%%%%% COMPUTE INITIAL GUESS FOR INVERSE BID FUNCTION:');
        display('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
        start  = (knotvecB(1)+knotvecB(2))/2;
        stop   = (knotvecB(end-1)+knotvecB(end))/2;
        dom    = bdom;
        indx   = find(dom>=start & dom<=stop);
        tempxi = dom(indx)+delta1;
        K      = length(knotvecB)-1;
        tempA  = [];
        tempb  = [];
        for i=1:K+2  %%%%This gives us monotonicity of the inverse bid function
            tempA = [tempA; zeros(1,i-1),1,-1,zeros(1,(K+3)-2-(i-1))];
            tempb = [tempb;0];
        end
        tempAeq   = [1, zeros(1,K+2)]; %%%%This enforces the boundary condition at the lower end
        tempbeq   = knotvecV(1);
        lb        = [];
        ub        = [];
        betaB0    = lsqlin(BasisCheckB(indx,:),tempxi,tempA,tempb,tempAeq,tempbeq,lb,ub);  %%%%This is the initial guess for FV parameters
        
        beta0 = [betaV0; betaB0; 10;10;10;10];
    elseif InitialGuessMethod==2
        load('InitialGuess.mat')
        tempv    = linspace(knotvecV0(1),knotvecV0(end),1000)';
        tempF    = BsplineEval3(knotvecV0,betaV0,tempv);
        tempBasis = BsplineBasis3(knotvecV,tempv,'off');
        tempA = [];
        tempb = [];
        for i=1:J+2  %%%%This gives us monotonicity of the CDF function
            tempA = [tempA; zeros(1,i-1),1,-1,zeros(1,(J+3)-2-(i-1))];
            tempb = [tempb;0];
        end
        tempAeq  = [1,zeros(1,J+2); zeros(1,J+2),1];  %%%%This gives us the constraint that the estimate must pass through 0 at the lower end
        tempbeq  = [0;1];
        lb       = [];
        ub       = [];
		betaV0 = lsqlin(tempBasis,tempF,tempA,tempb,tempAeq,tempbeq,lb,ub);  %%%%This is the initial guess for FV parameters
        
        
        display('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
        display('%%%%%%%%%% COMPUTE INITIAL GUESS FOR INVERSE BID FUNCTION:');
        display('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%');
        dom    = bdom;
        tempxi = BsplineEval3(knotvecB0,betaB0,dom);
        K      = length(knotvecB)-1;
        tempA  = [];
        tempb  = [];
        for i=1:K+2  %%%%This gives us monotonicity of the inverse bid function
            tempA = [tempA; zeros(1,i-1),1,-1,zeros(1,(K+3)-2-(i-1))];
            tempb = [tempb;0];
        end
        tempAeq   = [1, zeros(1,K+2)]; %%%%This enforces the boundary condition at the lower end
        tempbeq   = knotvecV(1);
        lb        = [];
        ub        = [];
        betaB0    = lsqlin(BasisCheckB,tempxi,tempA,tempb,tempAeq,tempbeq,lb,ub);  %%%%This is the initial guess for FV parameters
        
        beta0 = [betaV0; betaB0; 10*epsilon0];
        
    end
end
                                             
[beta,objfunvaltemp,exitflag,output] =...
    VDistSplineOpt(lambda,knotvecV,knotvecB,bstar,delta1,delta2,...
                                    bids,Fhat,BasisB,DerivB,bdom,BasisCheckB,DerivCheckB,...
                                    vdom,DerivCheckV,beta0,1,solver,penalty);

betaV = beta(1:J+3);
betaB = beta(J+4:end-4);
epsilon0 = beta(end-3);
epsilon1 = beta(end-2);
epsilon2 = beta(end-1);
epsilon3 = beta(end);
objfunval = objfunvaltemp;