function [beta,objfunval,exitflag,output] =...
    VDistSplineOpt(lambda,knotvecV,knotvecB,bstar,delta1,delta2,...
                         bids,Fhat,BasisB,DerivB,bdom,BasisCheckB,DerivCheckB,...
                         vdom,DerivCheckV,beta0,output,solver,penalty)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%

%%%% Note: In this file the structure of the vector beta is the following:
%%%% beta = [betaV;betaB1;betaB2;epsilon0;epsilon1;epsilon2], 
%%%% where dim(betaV)=(K+3)x1, dim(betaB1)=(K1+3)x1, dim(betaB2)=(K2+3)x1, and
%%%% epsilon0, epsilon1, epsilon2 are scalars.
if nargin<19
    penalty = [];
end

if nargin<18
    solver = 'knitro';
end
K  = length(knotvecB)-1;
J   = length(knotvecV)-1;
LB = length(BasisCheckB(:,1));

bdomdelta       = [(bdom(bdom<bstar+delta1)-delta1);...
                    bstar*ones(sum((bdom>=bstar+delta1)&(bdom<bstar+delta2)),1);...
                   (bdom(bdom>=bstar+delta2)-delta2)];
bdomdelta(bdomdelta<knotvecB(1)) = knotvecB(1); 
BasisCheckDelta = BsplineBasis3(knotvecB,bdomdelta,'off');

display('************************************************************************')
if strcmpi(solver,'knitro')
    display('****INITIATING KNITRO SOLVER FOR B-SPLINE CDF ESTIMATE****')
elseif strcmpi(solver,'fmincon')
    display('****INITIATING FMINCON SOLVER FOR B-SPLINE CDF ESTIMATE****')
end
display('************************************************************************')

A = [];
b = [];
for k=1:J+2  %%%%This loop gives us monotonicity of the private value density
    A = [A; zeros(1,k-1),1,-1,zeros(1,(J+3)-2-(k-1)), zeros(1,K+3), zeros(1,4)];
    b = [b;0];
end
for k=1:K+2  %%%%This loop gives us monotonicity of the inverse bid function
    A = [A; zeros(1,J+3), zeros(1,k-1),1,-1,zeros(1,(K+3)-2-(k-1)), zeros(1,4)];
    b = [b;0];
end

A        = [A; zeros(1,J+3), zeros(1,K+2),1, zeros(1,4)];  
                                               %%%%This adds the  constraint that xi cannot 
                                               %%%%be above the max private value \overline{v}                                           
b        = [b; (knotvecV(end))];
xibound=1;        
if xibound==1  %%%%This one constrains the inverse bid function to be above the 45-degree line
    A = [A;-[zeros(LB,J+3), BasisCheckB, zeros(LB,4)]];
    b = [b; -bdom;];
end

conBasisB = BsplineBasis3(knotvecB,knotvecB(1),'off');
Aeq      = [[1, zeros(1,J+2), zeros(1,K+3), zeros(1,4)];...
            [zeros(1,J+2), 1, zeros(1,K+3), zeros(1,4)];...
            [zeros(1,J+3), 1, zeros(1,K+2), zeros(1,4)]];  %%%%This gives us the constraint that the CDF 
                                              %%%%must pass through 0 at the lower end and 1 at the
                                              %%%%upper end.  The third row enforces the boundary
                                              %%%%condition for the inverse bid function.
beq      = [0;1;knotvecV(1)];                               
lb       = [-inf*ones(1,J+3), -inf*ones(1,K+3), 0, 0, 0, 0];  %%%%This enforces that the epsilons cannot be negative.
ub       = [];

xtolerance   = 1e-6;
funtolerance = 1e-6;
constrainttolerance = 1e-6;
nonlinear = @nonlcon;
constraintgradient = 'off';
disp = 'iter';


if nargin<7
    output=1;
end
if output==0 | strcmpi(output,'off')
    if strcmpi(solver,'knitro')
        opts = optimoptions('ktrlink','GradObj','off','GradConstr',constraintgradient,'tolx',...
                            xtolerance,'tolfun',funtolerance,'tolcon',constrainttolerance);
    elseif strcmpi(solver,'fmincon')
        opts = optimoptions('fmincon','UseParallel','always','Algorithm','active-set',...
                            'MaxFunEvals',40000,'MaxIter',500,'tolx',xtolerance,...
                            'tolfun',funtolerance,'tolcon',constrainttolerance);
    end
else
    if strcmpi(solver,'knitro')
        opts = optimoptions('ktrlink','Display',disp,...
                            'tolx',xtolerance,'tolfun',funtolerance,...
                            'tolcon',constrainttolerance);
    elseif strcmpi(solver,'fmincon')
        opts = optimoptions('fmincon','UseParallel','always','Algorithm','active-set','Display',disp,...
                            'MaxFunEvals',40000,'MaxIter',500,'tolx',...
                            xtolerance,'tolfun',funtolerance,'tolcon',constrainttolerance);
    end
end

if strcmpi(solver,'knitro')
    [beta,objfunval] = ktrlink(@NestedObjFun,beta0,A,b,Aeq,beq,lb,ub,nonlinear,opts);
elseif strcmpi(solver,'fmincon')
    [beta,objfunval,exitflag,output] = fmincon(@NestedObjFun,beta0,A,b,Aeq,beq,lb,ub,nonlinear,opts);
end


    function [objfn] = NestedObjFun(beta)
        if isempty(penalty)
            P0 = 10;  %%%%This is a penalty parameter that will control numerical error on the interval [\underline{b},\Delta_1]
            P1 = 200;  %%%%This is a penalty parameter that will control numerical error on the interval [\Delta_1, bstar]
            P2 = 200;  %%%%This is a penalty parameter that will control numerical error on the interval [bstar,\overline{b}]
            P3 = 200;
        else 
            P0 = penalty(1);
            P1 = penalty(2);
            P2 = penalty(3);
            P3 = penalty(4);
        end
        xi      = BasisB*beta(J+4:J+3+K+3);
        BasisV = BsplineBasis3(knotvecV,xi,'off');
        FV      = BasisV*beta(1:J+3);
        GB      = FV;  %%%%This is the parent distribution, as represented by the Bspline function
        H       = zeros(size(GB));
        normconst  = 1/(1-poissgencdf(1,lambda)); %%%%This normalization constant corrects for the fact that there are no auctions in the sample with fewer than 2 bidders.  This is needed in order to make the distribution of X be a valid CDF
        i=2;
        rho = nan(150,1);
        rho(1:i) = [0; normconst*poissgenpdf(i,lambda)];
        while i<lambda | rho(i)>=10^-10;
            H = H + rho(i)*( GB.^i + i*GB.^(i-1).*(1-GB) );  %%%%This is the CDF of the highest loser bid.
            i = i+1;
            rho(i) = normconst*poissgenpdf(i,lambda);
        end
        objfn   = sum( (Fhat - H).^2 ) + P0*beta(end-3) + P1*beta(end-2) + P2*beta(end-1) + P3*beta(end);
    end

    function [EquilibCond,ceq] = nonlcon(beta)
        Supnorm   = 1;
        L2norm    = 1-Supnorm;
        ceq       = []; 
        xi        = BasisCheckB*beta(J+4:J+3+K+3);
        xiDelta   = BasisCheckDelta*beta(J+4:J+3+K+3);
        xiprime   = DerivCheckB*beta(J+4:J+3+K+3);
        [BasisV,~,DerivV] = BsplineBasis3(knotvecV,xi,'off');
        BasisVDelta       = BsplineBasis3(knotvecV,xiDelta,'off');
        FV        = BasisV*beta(1:J+3);
        FVDelta   = BasisVDelta*beta(1:J+3);
        fV        = DerivV*beta(1:J+3);
        GB        = FV;
        GBDelta   = FVDelta;
        gB        = fV.*xiprime;
        %%%%Here we compute the Distribution and density functions for the
        %%%%highest competing bid:
        M         = (0:100)';
        probM     = poissgenpdf(M,lambda);
        normconst = 1/(1 - poissgencdf(1,lambda));
        weight    = probM.*normconst;
        Gm        = zeros(size(GB));
        GmDelta   = zeros(size(GBDelta));
        gm        = zeros(size(gB));
        for j=2:max(M)
            Gm      = Gm      + weight(j)*GB.^(M(j));
            GmDelta = GmDelta + weight(j)*GBDelta.^(M(j));
            gm      = gm      + weight(j)*(M(j)).*GB.^(M(j) - 1).*gB;
        end
        %%%%The next line contains the vector of nonlinear constraints.
        %%%%They say that, for any value of the parameter vector, the
        %%%%monotone hazard rate condition must be satisfied at each of the
        %%%%points in DerivCheckPts:
        indx0 = find(bdom<=knotvecB(1)+delta1,1,'last');
        indx1 = find(bdom<bstar+delta1,1,'last');
        indx2 = find(bdom<bstar+delta2,1,'last');
		R0 = (xi(1:indx0)-bdom(1:indx0)).*gm(1:indx0) - Gm(1:indx0);
        R1 = (xi(indx0+1:indx1)-bdom(indx0+1:indx1)).*gm(indx0+1:indx1)... 
                 - Gm(indx0+1:indx1) + GmDelta(indx0+1:indx1);
        R2 = (xi(indx1+1:indx2)-bdom(indx1+1:indx2)).*gm(indx1+1:indx2)... 
                 - Gm(indx1+1:indx2) + GmDelta(indx1+1:indx2);
        R3 = (xi(indx2+1:end)-bdom(indx2+1:end)).*gm(indx2+1:end)... 
                 - Gm(indx2+1:end) + GmDelta(indx2+1:end);
        if Supnorm ==1
        EquilibCond = [(R0-beta(end-3));(-R0-beta(end-3));...
                       (R1-beta(end-2));(-R1-beta(end-2));...
                       (R2-beta(end-1));(-R2-beta(end-1));...
                       (R3-beta(end));(-R3-beta(end))];
        elseif L2norm ==1
        EquilibCond = [(-sqrt(sum(R0.^2))-beta(end-3));...
                       (-sqrt(sum(R1.^2))-beta(end-2));...
                       (-sqrt(sum(R2.^2))-beta(end-1));...
                       (-sqrt(sum(R3.^2))-beta(end))];
        end
    end
end