%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
%%%%This file computes  cubicBspline basis functions and outputs some plots of
%%%%the basis functions and/or their derivatives.  Note that Bspline basis
%%%%functions are defined by a given knot vector.  The default behavior is
%%%%to use a uniformly spaced knot vector with no repeated knots (repeated
%%%%knots allow for the interpolant function to be less than C^2).  In
%%%%order to see the output with alternative knot vectors, try uncommenting
%%%%lines 29 or 30 below.  This is just a script file, but the computation
%%%%it performs are in function form in this toolbox: 

close all
clear all
clc
display('THIS FUNCTION COMPUTES CUBIC BSPLINE BASIS FUNCTIONS AND PLOTS THEM AND THEIR DERIVATIVES.')
display('NOTE THAT BSPLINE BASIS FUNCTIONS ARE DEFINED BY A KNOT VECTOR.  KNOTS ARE REPRESENTED IN')
display('PLOTS BY VERTICAL BLACK LINES.')
nargout=4;
D = 3;  %%%%This is the degree of the spline

plotfigs=1;
A = -1; %%%%This is the minimal domain point in the data
B = 1;  %%%%This is the maximal domain point in the data
N = 5;  %%%%This is the number of intervals defined by the data
knotvec = linspace(A,B,N+1)'; %%%%This is a hypothetical domain grid with uniformly spaced (non-repeating) knots.  
% knotvec = sort([knotvec(2);knotvec(2);knotvec(2);knotvec]); N=length(knotvec)-1; %%%%This line adds three repeats of the second knot (from the line above) into the knot vector to allow for a discontinuity.;

n = N+2*D;  %%%%This is the number of intervals where basis functions will be computed.
x = [A*ones(D,1); knotvec; B*ones(D,1)]; %%%%This is the extended data grid where basis functions will be computed

A = min(knotvec); %%%%This is the minimal value in the extended data grid
B = max(knotvec); %%%%This is the maximal value in the extended data grid
dom = linspace(A,B,1000)'; %%%%This is a domain grid on which the basis functions will be evaluated for graphing.

if min(dom)<A || max(dom)>B
    error('BsplineBasis3:ExtrapError','Cannot Extrapolate!!!!!!!!!!!!!!!')
end

%%%%In the following block we compute first order B-spline basis functions
%%%%and plot them:
order1 = 1;
d0=order1-1; %%%%d stands for "degree"
B0 = zeros(length(dom),n);
if plotfigs==1
    figure 
end
for i=1:N+d0
    indx = D+i-d0;
    if i==N+d0
        B0(:,indx) = (x(indx)<=dom).*(dom<=x(indx+1));
    else 
        B0(:,indx) = (x(indx)<=dom).*(dom<x(indx+1));
    end
    if plotfigs==1
        subplot(N+d0,1,i)
        if i==1
            title('FIRST ORDER (DEGREE ZERO) B-SPLINE BASIS FUNCTIONS')  
        end            
        hold on
        scatter(dom,B0(:,indx),'.')
        for j=2:n
            plot([x(j),x(j)],[-0.5,1.5],'k')
        end
        ylim([-0.2,1.2])
        xlim([A,B])
%         xlabel('DOMAIN POINTS')
    end
end

%%%%In the next block, we compute second order B-spline basis functions and
%%%%plot them:
order2 = 2;
d1 = order2-1; %%%%d stands for "degree"
B1 = zeros(length(dom),n);
if plotfigs==1
    figure
end
for i=1:N+d1
    indx = D+i-d1;
    if x(indx+d1)-x(indx)==0;
        B1(:,indx) = (x(indx+d1+1)-dom)/(x(indx+d1+1)-x(indx+1)).*B0(:,indx+1);
    elseif x(indx+d1+1)-x(indx+1)==0;
        B1(:,indx) = (dom-x(indx))/(x(indx+d1)-x(indx)).*B0(:,indx);
    else
        B1(:,indx) = (dom-x(indx))/(x(indx+d1)-x(indx)).*B0(:,indx) + (x(indx+d1+1)-dom)/(x(indx+d1+1)-x(indx+1)).*B0(:,indx+1);
    end

    if plotfigs==1
        subplot(N+d1,1,i)
        if i==1
            title('SECOND ORDER (DEGREE ONE) B-SPLINE BASIS FUNCTIONS')   
        end            
        hold on
        plot(dom,B1(:,indx))
        for j=2:n
            plot([x(j),x(j)],[-0.5,1.5],'k')
        end
        ylim([-0.2,1.2])
        xlim([A,B])
    end
end

%%%%In the next block, we compute third order B-spline basis functions,
%%%%plus derivatives (these will be used later to compute second 
%%%%derivatives of the fourth order functions), and we plot the basis 
%%%%functions.
order3 = 3;
d2 = order3-1; %%%%d stands for "degree"
B2 = zeros(length(dom),n);
if plotfigs==1
    figure   
end
for i=1:N+d2
    indx = D+i-d2;
    if x(indx+d2)-x(indx)==0;
        B2(:,indx) = (x(indx+d2+1)-dom)/(x(indx+d2+1)-x(indx+1)).*B1(:,indx+1);
    elseif x(indx+d2+1)-x(indx+1)==0;
        B2(:,indx) = (dom-x(indx))/(x(indx+d2)-x(indx)).*B1(:,indx);
    else
        B2(:,indx) = (dom-x(indx))/(x(indx+d2)-x(indx)).*B1(:,indx) + (x(indx+d2+1)-dom)/(x(indx+d2+1)-x(indx+1)).*B1(:,indx+1);
    end
    
    if plotfigs==1
        subplot(N+d2,1,i)
        if i==1
            title('THIRD ORDER (DEGREE TWO) B-SPLINE BASIS FUNCTIONS')
        end
        hold on
        plot(dom,B2(:,indx))
        for j=2:n
            plot([x(j),x(j)],[-0.5,1.5],'k')
        end
        ylim([-0.2,1.2])
        xlim([A,B])
    end
end

%%%%In this next block we compute fourth order B-spline basis functons, and
%%%%their first and second derviatives.  We then plot the basis functions:
order4 = 4;
d3 = order4-1; %%%%d stands for "degree"
B3 = zeros(length(dom),n);
if nargout>=3
    Yprime = zeros(size(B3)); %%%%The variable Z here will contain the derivatives of the cubic Bspline basis functions, evaluated at the points in dom
end
if nargout==4
    Ydblprime = zeros(size(B3));
end
if plotfigs==1
    figure
end
for i=1:N+d3
    indx = D+i-d3;
    if x(indx+d3)-x(indx)==0;
        B3(:,indx) = (x(indx+d3+1)-dom)/(x(indx+d3+1)-x(indx+1)).*B2(:,indx+1);
        if nargout>=3
            Yprime(:,indx)  = (-d3/(x(indx+d3+1)-x(indx+1)))*B2(:,indx+1);
        end
        if nargout==4
            %%%%First we need to compute dB2ii, 
            %%%%the derivative of B2(:,indx+1):
            if x(indx+1+d2)-x(indx+1)==0
                dB2ii = (-d2/(x(indx+1+d2+1)-x(indx+1+1)))*B1(:,indx+1+1);
            else
                dB2ii = d2*( B1(:,indx+1)/(x(indx+1+d2)-x(indx+1)) - B1(:,indx+1+1)/(x(indx+1+d2+1)-x(indx+1+1)) );
            end
            Ydblprime(:,indx)  = (-d3/(x(indx+d3+1)-x(indx+1)))*dB2ii;
        end
    elseif x(indx+d3+1)-x(indx+1)==0;
        B3(:,indx) = (dom-x(indx))/(x(indx+d3)-x(indx)).*B2(:,indx);
        if nargout>=3
            Yprime(:,indx)  = (d3/(x(indx+d3)-x(indx)))*B2(:,indx);
        end
        if nargout==4
            %%%%First we need to compute dB2i, 
            %%%%the derivative of B2(:,indx):
            if x(indx+d2+1)-x(indx+1)==0
                dB2i = (d2/(x(indx+d2)-x(indx)))*B1(:,indx);
            else
                dB2i = d2*( B1(:,indx)/(x(indx+d2)-x(indx)) - B1(:,indx+1)/(x(indx+d2+1)-x(indx+1)) );
            end
            Ydblprime(:,indx)  = (d3/(x(indx+d3)-x(indx)))*dB2i;
        end
    else
        B3(:,indx) = (dom-x(indx))/(x(indx+d3)-x(indx)).*B2(:,indx) + (x(indx+d3+1)-dom)/(x(indx+d3+1)-x(indx+1)).*B2(:,indx+1);
        if nargout>=3
            Yprime(:,indx)  = d3*( B2(:,indx)/(x(indx+d3)-x(indx)) - B2(:,indx+1)/(x(indx+d3+1)-x(indx+1)) );
        end
        if nargout==4
            %%%%First we need to compute dB2i, 
            %%%%the derivative of B2(:,indx), 
            %%%%as well as dB2ii, 
            %%%%the derivative of B2(:,indx+1):
            if x(indx+d2)-x(indx)==0 
                dB2i  = -d2*B1(:,indx+1)/(x(indx+d2+1)-x(indx+1));
            else
                dB2i  = d2*( B1(:,indx)/(x(indx+d2)-x(indx)) - B1(:,indx+1)/(x(indx+d2+1)-x(indx+1)) );
            end
            if x(indx+1+d2+1)-x(indx+1+1)==0
                dB2ii = d2*B1(:,indx+1)/(x(indx+1+d2)-x(indx+1));
            else
                dB2ii = d2*( B1(:,indx+1)/(x(indx+1+d2)-x(indx+1)) - B1(:,indx+1+1)/(x(indx+1+d2+1)-x(indx+1+1)) );
            end
            
            Ydblprime(:,indx)  = d3*( dB2i/(x(indx+d3)-x(indx)) - dB2ii/(x(indx+d3+1)-x(indx+1)));
        end
    end

    if plotfigs==1
        subplot(N+d3,1,i)
        if i==1
            title('FOURTH ORDER (DEGREE THREE) B-SPLINE BASIS FUNCTIONS')   
        end
        hold on
        plot(dom,B3(:,indx))
        for j=2:n
            plot([x(j),x(j)],[-0.5,1.5],'k')
        end
        ylim([-0.2,1.2])
        xlim([A,B])
    end
end

%%%%Here we plot all of the basis functions on the same axes:
if plotfigs==1
    figure
    title('THIRD DEGREE B-SPLINE BASIS FUNCTIONS')   
    hold on
    plot(dom,sum(B3(:,1:N+D),2),'--b','LineWidth',2)
    for i=1:N+D;
        plot(dom,B3(:,i),'r','LineWidth',3);
        if i<=N
            plot([knotvec(i),knotvec(i)],[-0.5,1.5],'k')
        end
        ylim([-0.25,1.25])
        xlim([A,B])
    end
    xlabel('NOTE: the dashed line is the sum of all basis functions.')
end

Y      = B3(:,1:N+D);
if nargout >=2
    Z      = diag(ones(N+D,1))+diag(ones(N+D-1,1),1)+diag(ones(N+D-1,1),-1)+diag(ones(N+D-2,1),-2);
    Z      = Z(:,2:N+1);
end
if nargout>=3
    Yprime = Yprime(:,1:N+D);
end
if nargout==4
    Ydblprime = Ydblprime(:,1:N+D);
end

%%%%In this final block of code, we plot each basis function with it's
%%%%first and second derivatives:
if plotfigs==1
    domnumeric       = 0.5*dom(1:end-1)+0.5*dom(2:end);
    Yprimenumeric    = zeros(length(dom)-1,N+D);
    Ydblprimenumeric = zeros(length(dom)-1,N+D);
    for i=1:N+D
        Yprimenumeric(:,i)    = (Y(2:end,i)-Y(1:end-1,i))./(dom(2:end)-dom(1:end-1));
        Ydblprimenumeric(:,i) = (Yprime(2:end,i)-Yprime(1:end-1,i))./(dom(2:end)-dom(1:end-1));
        figure
        title(strcat('B-Spline Basis Function',' ',num2str(i),' of',' ',num2str(N+D),' w/ First and Second Derivatives'))
        hold on
        plot(dom,Y(:,i),...
             dom,Yprime(:,i),...
             domnumeric,Yprimenumeric(:,i),'--',...
             dom,Ydblprime(:,i),...
             domnumeric,Ydblprimenumeric(:,i),'--','LineWidth',3)
        legend('BASIS FUNCTION','ANALYTIC DERIVATIVE','NUMERIC DERIVATIVE','ANALYTIC 2ND DERIVATIVE','NUMERIC 2ND DERIVATIVE')
        for j=2:N
            plot([knotvec(j),knotvec(j)],[1.2*min(Ydblprime(:,i)),1.2*max(Ydblprime(:,i))],'k')
        end
        ylim([1.1*min(Ydblprime(:,i)),1.1*max(Ydblprime(:,i))])
    end
end

