function y=twblcdf(x,l,u,a,b)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
%TWBLCDF(x,l,u,a,b) calculates a truncated Weibull cdf with scale parameter 
%a, shape parameter b, lower bound l and upper bound u.  The minimal number
%of arguments is 3; if a/b are left unspecified, then their default values 
%of 1 will be assigned.  Components of x that lie outside the interval
%[l,u] will be assigned functional values of zero or one.
%
%When b=1, this function computes a truncated exponential cdf with
%parameter a.
% 
%See also TLOGNCDF, TNORMCDF, TRAYLCDF, TLOGNPDF, TNORMPDF, TRAYLPDF, 
%TWBLPDF

%First, make sure that bounds have been properly specified:
if nargin < 3
    error('YOU MUST AT LEAST SPECIFY AN INPUT AND LOWER/UPPER BOUNDS!');
end
%Next, return a warning message if the lower bound is set to be less than
%the natural lower bound of 0
if l<0
    display('WARNING: a negative lower bound on the Weibull is trivially satisfied');
    display('Lower bound has been redefined as "0".');
    l=0;
end

if nargin < 4
    a = 1;
end
if nargin < 5
    b = 1;
end

if l==0 & isinf(u)          % if the bounds supplied are the natural bounds
    y=wblcdf(x,a,b)
elseif l==0 & isfinite(u)   % if the right tail only is truncated
    y=wblcdf(x,a,b)*(1/wblcdf(u,a,b));
elseif l>0 & isinf(u)       % if the left tail only is truncated
    y=(wblcdf(x,a,b) - wblcdf(l,a,b))*(1/(1-wblcdf(l,a,b)));
elseif l>0 & isfinite(u)    %if both tails are truncated
    y=(wblcdf(x,a,b) - wblcdf(l,a,b))*(1/(wblcdf(u,a,b)-wblcdf(l,a,b)));
end

y(find(x<=l))=0;            % for functional values outside the truncated domain
y(find(x>=u))=1;
