%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% main driver file to estimate correctly-specified electronic auction model
% using eBay data

clear all
close all
clc

solver = 'fmincon';
if strcmpi(solver,'fmincon')
     matlabpool open local
end

for zzzz=11%5:16
tic
% parameters for figure properties
lwidth = 2;
fsize = 14;
set(0,'defaulttextinterpreter','latex','Defaulttextfontsize',fsize);
tic

J  = zzzz;  %%%%This is going to be the number of subintervals in the knot vector for FV, or in 
            %%%%other words, the number of parameters for FV will be J+3.
K = 47;     %%%%This is the number of subintervals xi, not counting the extra 4 subintervals that will be inserted just above bstar
load EbayAllObsSim30Trim;  %%%%This data file contains the original sample and 1000 bootstrapped samples
load ProbNtildegN;     %%%%This data file contains conditional probabilities for n given \tilde{n}
bstar = 250;           %%%%This is the transition point where the bid increment changes
delta1 = 2.5;          %%%%This is the bid increment below the transition point
delta2 = 5;            %%%%This is the bid increment above the transition point

% obsdata comes from EbayAllObsSim30 and contains the original data. Column
% 1 contains observed N, column 2 contains the highest losing bid, and
% column 3 contains the winning bid, but only for auctions that triggered
% the first price rule
obsdata = sortrows(obsdata(obsdata(:,2)<=400,:),2);
N = obsdata(:,1);
bids = obsdata(:,2);
% this controls Type I error for the confidence bounds.
alphaover2 = 0.025;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%   FIRST WE ESTIMATE THE POISSON PARAMETER                                  %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
display('NOW COMPUTING THE GENERALIZED POISSON PARAMETERS BY NAIVELY TAKING OBSERVED N AS ACTUAL N...')
display('')
[lambdaNAIVE,lsqobjfnvalNAIVE,exitflagNAIVE,outputNAIVE] =... 
    FilterOptNaive(obsdata,[7,0.1],1,'fmincon')
display('NOW COMPUTING THE GENERALIZED POISSON PARAMETERS BY ACCOUNTING FOR THE NATURAL FILTER PROCESS...')
display('')
[lambdaPE,objfunvalLambdaPE,exitflagLambdaPE,outputLambdaPE]...
    = FilterOpt(obsdata,ProbNgNtilde,[7,0.1],1,'fmincon') %%%%Here we estimate the poisson arrival rate of bidders


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% NEXT WE BUILD SOME PRELIMINARY INPUTS TO THE EMPIRICAL CRITERION FUNCTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
L         = 500;  %%%%This will be the number of points in the grids where contraint conditions will be checked.
knotvecV  = linspace(0,1,J)';
[Ftemp,Xtemp] = ksdensity(bids,linspace(min(bids),max(bids),1000),'function','cdf');
knotvecV(1)   = min(bids);
knotvecV(end) = max(bids)+delta2*2;
knotvecV(2:end-1) = interp1q(Ftemp',Xtemp',knotvecV(2:end-1));
knotvecV      = sort([knotvecV;0.5*(knotvecV(end-1)+knotvecV(end))])
J = length(knotvecV)-1;
clear Ftemp Xtemp


vdom       = linspace(knotvecV(1),knotvecV(end),L)';
transition = (delta2-delta1)/4;   %%%%The program inserts extra knots into the transition region just above bstar for added flexibility.
knotsbelow = round((K+1)*(bstar - min(bids))/(max(bids)-min(bids)));  %%%%This is the fraction of the knots that should go below bstar.
knotvecB   = [linspace(min(bids),bstar+delta1-4*transition,knotsbelow)';...
              (bstar+delta1-3*transition:transition:bstar+delta1+7*transition)';...
              linspace(bstar+delta1+8*transition,max(bids),(K+1)-knotsbelow)'];
K          = length(knotvecB)-1;
bdom       = linspace(knotvecB(1),knotvecB(end),L)';

T     = length(bids);
Fhat  = zeros(length(bids),1);
for i=1:length(bids)
    Fhat(i) = sum(bids<=bids(i))/T;
end

beta0   = [];
penalty = [];

[betaVPE,betaBPE,epsilon0PE,epsilon1PE,epsilon2PE,epsilon3PE,objfunvalmainPE,exitflagmainPE,outputmainPE] =... 
    VDistSplineEstim(bids,lambdaPE,delta1,delta2,bstar,knotvecV,knotvecB,...
                               vdom,bdom,Fhat,L,solver,beta0,penalty);
OptimizationTime = toc/60


%%                           
figure(1)
hold on
[BasisB,~,DerivB] = BsplineBasis3(knotvecB,bdom);
plot(bdom,bdom,'k','LineWidth',1.5)
plot(bdom,BasisB*betaBPE,'r','LineWidth',3)

figure(2)
[BasisV,~,DerivV] = BsplineBasis3(knotvecV,vdom);
subplot(2,1,1)
hold on
plot(vdom,BasisV*betaVPE,'r','LineWidth',3)
subplot(2,1,2)
hold on
plot(vdom,DerivV*betaVPE,'r','LineWidth',3)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% PLOT HIGHEST LOSER DISTRIBUTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
xi      = BasisB*betaBPE;
xiprime = DerivB*betaBPE;
[BasisV,~,DerivV] = BsplineBasis3(knotvecV,xi,'off');
FV      = BasisV*betaVPE;
fV      = DerivV*betaVPE;
GB      = FV;  %%%%This is the parent distribution, as represented by the Bspline function
gB      = fV.*xiprime;
H       = zeros(size(GB));  %%%%This is the distribution of the second order statistic, X
% h       = zeros(size(GB));
normconst  = 1/(1-poissgencdf(1,lambdaPE)); %%%%This normalization constant corrects for the fact that there are no auctions in the sample with fewer than 2 bidders.  This is needed in order to make the distribution of X be a valid CDF

i=2;
rho = nan(150,1);
rho(1:i) = [0; normconst*poissgenpdf(i,lambdaPE)];
while i<lambdaPE | rho(i)>=10^-10;
    H = H + rho(i)*( GB.^i + i*GB.^(i-1).*(1-GB) );  %%%%This is the CDF of the highest loser bid.
    i = i+1;
    rho(i) = normconst*poissgenpdf(i,lambdaPE);
end


figure(3)
hold on
[Hplot,bplot] = ecdf(bids);
stairs(bplot,Hplot,'--k','LineWidth',3)
plot(bdom,H,'r','LineWidth',3)
legend('EMPIRICAL $\hat{H}(b)$','FITTED $H(b;\hat{\lambda},\hat{\mu},\hat{\alpha})$','Location','SouthEast')
xlabel('HIGHEST LOSER BIDS')
ylabel('CDF')
box on
grid on


[BasisCheckB,~,DerivCheckB] = BsplineBasis3(knotvecB,bdom,'off');
[l,w]                       = size(DerivCheckB);

K  = length(knotvecB)-1;
J   = length(knotvecV)-1;
LB = length(BasisCheckB(:,1));

bdomdelta       = [(bdom(bdom<bstar+delta1)-delta1);...
                    bstar*ones(sum((bdom>=bstar+delta1)&(bdom<bstar+delta2)),1);...
                   (bdom(bdom>=bstar+delta2)-delta2)];
bdomdelta(bdomdelta<knotvecB(1)) = knotvecB(1); 
BasisCheckDelta = BsplineBasis3(knotvecB,bdomdelta,'off');

xi        = BasisCheckB*betaBPE;
xiDelta   = BasisCheckDelta*betaBPE;
xiprime   = DerivCheckB*betaBPE;
[BasisV,~,DerivV] = BsplineBasis3(knotvecV,xi,'off');
BasisVDelta       = BsplineBasis3(knotvecV,xiDelta,'off');
FV        = BasisV*betaVPE;
FVDelta   = BasisVDelta*betaVPE;
fV        = DerivV*betaVPE;
GB        = FV;
GBDelta   = FVDelta;
gB        = fV.*xiprime;
%%%%Here we compute the Distribution and density functions for the
%%%%highest competing bid:
M         = (0:100)';
probM     = poissgenpdf(M,lambdaPE);
normconst = 1/(1 - poissgencdf(1,lambdaPE));
weight    = probM.*normconst;
Gm        = zeros(size(GB));
GmDelta   = zeros(size(GBDelta));
gm        = zeros(size(gB));
for j=2:max(M)
    Gm      = Gm      + weight(j)*GB.^(M(j));
    GmDelta = GmDelta + weight(j)*GBDelta.^(M(j));
    gm      = gm      + weight(j)*(M(j)).*GB.^(M(j) - 1).*gB;
end
%%%%The next line contains the vector of nonlinear constraints.
%%%%They say that, for any value of the parameter vector, the
%%%%monotone hazard rate condition must be satisfied at each of the
%%%%points in DerivCheckPts:
indx0 = find(bdom<=knotvecB(1)+delta1,1,'last');
indx1 = find(bdom<bstar+delta1,1,'last');
indx2 = find(bdom<bstar+delta2,1,'last');
R0 = (xi(1:indx0)-bdom(1:indx0)).*gm(1:indx0) - Gm(1:indx0);
R1 = (xi(indx0+1:indx1)-bdom(indx0+1:indx1)).*gm(indx0+1:indx1)... 
         - Gm(indx0+1:indx1) + GmDelta(indx0+1:indx1);
R2 = (xi(indx1+1:indx2)-bdom(indx1+1:indx2)).*gm(indx1+1:indx2)... 
         - Gm(indx1+1:indx2) + GmDelta(indx1+1:indx2);
R3 = (xi(indx2+1:end)-bdom(indx2+1:end)).*gm(indx2+1:end)... 
         - Gm(indx2+1:end) + GmDelta(indx2+1:end);
R = [R0;R1;R2;R3];


figure(4)
hold on
plot(bdom,zeros(size(bdom)),'k')
plot(bdom,R./xi,'r','LineWidth',3)
legend('ZERO LINE','RELATIVE ERROR: $R(b)/\hat{\xi}(b;\hat{\alpha})$','Location','SouthEast')
xlabel('BIDS')
ylabel('APPROXIMATION ERROR')
xlim([min(bids),max(bids)])
box on
grid on

beta0 = [betaVPE;betaBPE;epsilon0PE;epsilon1PE;epsilon2PE;epsilon3PE];
save(strcat('beta0_Order',num2str(zzzz),'.mat'),'beta0','lambdaPE','objfunvalmainPE')
end
%%
SSR  = nan(12,2); 
eps0 = nan(12,1);
eps1 = nan(12,1);
eps2 = nan(12,1);
eps3 = nan(12,1);
for j=5:16;
    j 
    load(strcat('beta0_Order',num2str(j),'.mat')); 
    SSR(j,1) = j;
    SSR(j,2)  = objfunvalmainPE-10*sum(beta0(end-3:end-1)) -500*beta0(end); 
    eps0(j,1) = beta0(end-3);
    eps1(j,1) = beta0(end-2);
    eps2(j,1) = beta0(end-1);
    eps3(j,1) = beta0(end);
end;
SSR
eps0
eps1
eps2
eps3

%%
S = length(simdata);
BOOTSTRAP = 0;

if BOOTSTRAP==1;
%%
% number of simulations
% initialize some variables
lambda          = nan(2,S);
objfunvalLambda = nan(S,1);  
exitflagLambda  = nan(S,1);
outputLambda    = cell(S,1);
betaV           = nan(J+3,S);
betaB           = nan(K+3,S);
epsilon0        = nan(S,1);
epsilon1        = nan(S,1);
epsilon2        = nan(S,1);
epsilon3        = nan(S,1);
objfunvalMain   = nan(S,1);
exitflagMain    = nan(S,1);
outputMain      = cell(S,1);
%%
% parfor s=1:S

for s=1:S
                           
    display('##########################################################')
    display('##########################################################')
	display(strcat('########################### SIMULATION NUMBER',num2str(s)))
    display('##########################################################')
    display('##########################################################')
	data = simdata{s};
	% compute bid function by interpolating (pseudo value,bid) pairs
    data = sortrows(data,2);
	N = data(:,1);
	bids = data(:,2);
    T     = length(bids);
    Fhat  = zeros(length(bids),1);
    for i=1:length(bids)
        Fhat(i) = sum(bids<=bids(i))/T;
    end
	
	% recover latent valuations
	[lambdatemp,objfunval,exitflag,output]...
        = KnitroLSqLambdaOptSimpleFilterGen(data,ProbNgNtilde,lambdaPE,1,'fmincon');
    lambda(:,s)        = lambdatemp';
    objfunvalLambda(s) = objfunval;
    exitflagLambda(s)  = exitflag;
    outputLambda{s}    = output;
    
    beta0 = [betaVPE;betaBPE;epsilon0PE;epsilon1PE;epsilon2PE;epsilon3PE];
    
    [betaVtemp,betaBtemp,epsilon0temp,epsilon1temp,epsilon2temp,epsilon3temp,objfunval,exitflag,output] =... 
    VDistSplineEstim(bids,lambdatemp,delta1,delta2,bstar,knotvecV,knotvecB,...
                               vdom,bdom,Fhat,L,solver,beta0,[]);
    betaV(:,s)       = betaVtemp;
    betaB(:,s)       = betaBtemp;
    epsilon0(s)      = epsilon0temp;
    epsilon1(s)      = epsilon1temp;
    epsilon2(s)      = epsilon2temp;
    epsilon3(s)      = epsilon3temp;
    objfunvalMain(s) = objfunval;
    exitflagMain(s)  = exitflag;
    outputMain{s}    = output;
    
end

save('BootstrapsOrder11_1_1000FINAL_Pequals10-200-200-200.mat','lambda','objfunvalLambda','exitflagLambda','outputLambda',...
                           'lambdaPE','objfunvalLambdaPE','exitflagLambdaPE','outputLambdaPE',...
                           'betaV','betaB','epsilon0','epsilon1','epsilon2','epsilon3','objfunvalMain',...
                           'exitflagMain','outputMain',...
                           'betaVPE','betaBPE','epsilon0PE','epsilon1PE','epsilon2PE','objfunvalmainPE',...
                           'exitflagmainPE','outputmainPE',...
                           'knotvecV','knotvecB','K','J','bstar','delta1','delta2');
end
%%

load('BootstrapsOrder11_1_1000FINAL_Pequals10-200-200-200.mat')

[Basistest,~,Derivtest] = BsplineBasis3(knotvecV,vdom,'off');
xiBasistest = BsplineBasis3(knotvecB,bdom,'off');

FVtest = zeros(length(vdom),S); 
fVtest = zeros(length(vdom),S);
xitest = zeros(length(bdom),S);

parfor i=1:S; 
    FVtest(:,i) = Basistest*betaV(:,i); 
    fVtest(:,i) = Derivtest*betaV(:,i); 
    xitest(:,i) = xiBasistest*betaB(:,i); 
end

FVtest(:,exitflagMain==0) =[]; 
fVtest(:,exitflagMain==0) =[];
xitest(:,exitflagMain==0) =[];

fVtest(:,isnan(FVtest(1,:))) = [];
xitest(:,isnan(FVtest(1,:))) = [];
FVtest(:,isnan(FVtest(1,:))) = [];

FVtest = sort(FVtest,2);
fVtest = sort(fVtest,2);
xitest = sort(xitest,2);


figure(1)
hold on
lowercb = round(length(xitest(1,:))*alphaover2);
uppercb = round(length(xitest(1,:))*(1-alphaover2));
plot(bdom,xitest(:,lowercb),'--k',bdom,xitest(:,uppercb),'--k','LineWidth',1.5)
if alphaover2==0.025
    legend('$45^\circ$-LINE','$\hat{\xi}(b;\hat{\mu})$ POINT ESTIMATE','95\% CONFIDENCE BOUNDS','Location','SouthEast')
elseif alphaover2==0.05
    legend('$45^\circ$-LINE','$\hat{\xi}(b;\hat{\mu})$ POINT ESTIMATE','90\% CONFIDENCE BOUNDS','Location','SouthEast')
elseif alphaover2==0.005
    legend('$45^\circ$-LINE','$\hat{\xi}(b;\hat{\mu})$ POINT ESTIMATE','99\% CONFIDENCE BOUNDS','Location','SouthEast')
end
xlim([min(bids),max(bids)])
for j=1:length(knotvecB)
    plot([knotvecB(j),knotvecB(j)],[200,220],'k')
end
xlabel('$B$ (BIDS)')
ylabel('$V$ (PRIVATE VALUES)')
box on

figure(2)
subplot(2,1,1)
hold on
lowercb = round(length(FVtest(1,:))*alphaover2);
uppercf = round(length(FVtest(1,:))*(1-alphaover2));
plot(vdom,FVtest(:,lowercb),'--k',vdom,FVtest(:,uppercb),'--k','LineWidth',1.5);
if alphaover2==0.025
    legend('$\hat{F}_{V}(v;\hat{\mu})$ POINT ESTIMATE','95\% CONFIDENCE BOUNDS','Location','SouthEast')
elseif alphaover2==0.05
    legend('$\hat{F}_{V}(v;\hat{\mu})$ POINT ESTIMATE','90\% CONFIDENCE BOUNDS','Location','SouthEast')
elseif alphaover2==0.005
    legend('$\hat{F}_{V}(v;\hat{\mu})$ POINT ESTIMATE','99\% CONFIDENCE BOUNDS','Location','SouthEast')
end
for i=1:length(knotvecV)
    plot([knotvecV(i),knotvecV(i)],[0,1],'k')
end
xlim([knotvecV(1),knotvecV(end)])
box on
% grid on
xlabel('PRIVATE VALUES')
ylabel('CDF')
subplot(2,1,2); 
hold on; 
plot(vdom,fVtest(:,lowercb),'--k',vdom,fVtest(:,uppercb),'--k','LineWidth',1.5)
if alphaover2==0.025
    legend('$\hat{f}_{V}(v;\hat{\mu})$ POINT ESTIMATE','95\% CONFIDENCE BOUNDS','Location','NorthEast')
elseif alphaover2==0.05
    legend('$\hat{f}_{V}(v;\hat{\mu})$ POINT ESTIMATE','90\% CONFIDENCE BOUNDS','Location','NorthEast')
elseif alphaover2==0.005
    legend('$\hat{f}_{V}(v;\hat{\mu})$ POINT ESTIMATE','99\% CONFIDENCE BOUNDS','Location','NorthEast')
end
for i=1:length(knotvecV)
    plot([knotvecV(i),knotvecV(i)],[0,max(fVtest(:,uppercb))*1.2],'k')
end
xlim([knotvecV(1),knotvecV(end)])
ylim([0,max(fVtest(:,uppercb))*1.2])
xlabel('PRIVATE VALUES')
ylabel('PDF')
box on


figure(77)
subplot(2,1,1)
hold on
[BasisB,~,DerivB] = BsplineBasis3(knotvecB,bdom);
plot(bdom,bdom,'k','LineWidth',1.5)
plot(bdom,BasisB*betaBPE,'r','LineWidth',3)
plot(bdom,xitest(:,lowercb),'--k',bdom,xitest(:,uppercb),'--k','LineWidth',1.5)
if alphaover2==0.025
    legend('$45^\circ$-LINE','$\hat{\xi}(b;\hat{\alpha})$ ESTIMATE','95\% CB','Location','Best')
elseif alphaover2==0.05
    legend('$45^\circ$-LINE','$\hat{\xi}(b;\hat{\alpha})$ ESTIMATE','90\% CB','Location','Best')
elseif alphaover2==0.005
    legend('$45^\circ$-LINE','$\hat{\xi}(b;\hat{\alpha})$ ESTIMATE','99\% CB','Location','Best')
end
xlim([min(bids),max(bids)])
for j=1:length(knotvecB)
    plot([knotvecB(j),knotvecB(j)],[200,250],'k')
end
xlabel('$B$','FontSize',20)
ylabel('$V$','FontSize',20)
box on
subplot(2,1,2)
hold on
plot(bdom,zeros(size(bdom)),'k','LineWidth',1.5)
plot(bdom,R./xi,'r','LineWidth',3)
legend('ZERO LINE','RELATIVE ERROR: $R(b)/\hat{\xi}(b;\hat{\alpha})$','Location','SouthEast')
xlabel('$B$','FontSize',20)
ylabel('APPROXIMATION ERROR','FontSize',20)
xlim([min(bids),max(bids)])
box on
grid on
%%

% %%%%------------------------------------------------------------------------------------------------
% %%%%------------------------------------------------------------------------------------------------
% %%%% Now comparing our real-time ODE solution to a standard Galerkin solution, holding F_V fixed:
% %%%%------------------------------------------------------------------------------------------------
% %%%%------------------------------------------------------------------------------------------------
% %%
domB = linspace(knotvecB(1),knotvecB(end),1000)';
%%%%IMPORTANT NOTE: it is extremely important that the user provide a good
%%%%initial guess in order to ensure good behavior of the nonlinear solver
%%%%that will find the zeros of our Galerkin residuals.  Since we know that
%%%%the runtime (over-fitted) Galerkin solution adheres to the constraints
%%%%we need, here we use the point estimmates to find a good initial guess:
Ktest = K*5;
transition = (delta2-delta1)/4;   %%%%The program inserts extra knots into the transition region just above bstar for added flexibility.
knotsbelow = round((Ktest+1)*(bstar - min(bids))/(max(bids)-min(bids)));  %%%%This is the fraction of the knots that should go below bstar.
knotvecBtest   = [linspace(min(bids),bstar+delta1-4*transition,knotsbelow)';...
    (bstar+delta1-3*transition:transition:bstar+delta1+7*transition)';...
    linspace(bstar+delta1+8*transition,max(bids),(Ktest+1)-knotsbelow)'];
Ktest = length(knotvecBtest)-1;
Basistest = BsplineBasis3(knotvecBtest,domB,'off');
funvals   = BsplineEval3(knotvecB,betaBPE,domB);
A = [1,-1,zeros(1,Ktest+1)];
d = 0;
for i=2:Ktest+2
    A = [A; zeros(1,i-1),1,-1,zeros(1,Ktest+1-(i-1))]; %#ok
    d = [d; 0]; %#ok
end
Aeq = Basistest(1,:);
deq = knotvecV(1);
betaBtest0  = lsqlin(Basistest,funvals,A,d,Aeq,deq);  %%%%THIS IS OUR INITIAL GUESS FOR THE SOLVER.

display('')
display('%%%%---------------------------------------------')
display('%%%%---------------------------------------------')
display(strcat('%%%%  NOW COMPUTING GALERKIN ODE SOLUTION, WITH_',num2str(Ktest),' KNOTS, AND HOLDING \hat{F}_V FIXED...'));
display('%%%%---------------------------------------------')
display('%%%%---------------------------------------------')
betaBtest = GalerkinSolveEA(lambdaPE,[betaVPE;betaBtest0],knotvecV,knotvecBtest,delta1,delta2,bstar);

figure(314159)
pvPE = BsplineEval3(knotvecB,betaBPE,domB);
pvtest = BsplineEval3(knotvecBtest,betaBtest,domB);
subplot(2,1,1)
hold on
plot(domB,zeros(length(domB),1),domB,(pvPE-pvtest),'LineWidth',3)
xlabel('$B$ (BIDS)')
ylabel('SOLUTION METHOD DIFFERENCE')
legend('ZERO LINE','(RUNTIME (overfitted) GALERKIN ODE SOLUTION)-(EX-POST (square) GALERKIN ODE SOLUTION)')
grid on
box on
subplot(2,1,2)
hold on
plot(domB,domB,'--k','LineWidth',1.5)
plot(domB,pvPE,'r',domB,pvtest,'c','LineWidth',4)
xlabel('$B$ (BIDS)')
ylabel('SOLUTION METHOD LEVELS')
legend('ZERO LINE','$\hat{\xi}(b;\hat{\alpha})$','\tilde{\xi}(b;\tilde{\alpha})$')
box on
grid on

%%%%------------------------------------------------------------------------------------------------
%%%%------------------------------------------------------------------------------------------------
%%
display('')
display('%%%%---------------------------------------------')
display('%%%%---------------------------------------------')
display('%%%%  Doing Revenue Calculations now...')
display('%%%%---------------------------------------------')
display('%%%%---------------------------------------------')
tic

revcalc = 1;
if revcalc == 1
	% revenue calculations at point estimate
	[ExprevPE,hypIncExprevPE,hypDecExprevPE,optimalrPE] = RevenueCalculations(betaVPE,knotvecV,lambdaPE);
	maxExprevPE = ComputeExpRev(betaVPE,knotvecV,lambdaPE,optimalrPE);
	ngrid = 1000;
	rvect = linspace(knotvecV(1),knotvecV(end),ngrid);
	figure
	plot(rvect,ExprevPE,'LineWidth',3)
	xlabel('$r$ (RESERVE PRICE)','fontsize',14)
	ylabel('$R$ (EXPECTED REVENUES)','fontsize',14)
	hold all
	plot(rvect,hypIncExprevPE,'g:','LineWidth',3)
	plot(rvect,hypDecExprevPE,'r-.','LineWidth',3)
	[maxrev,maxrevind] = max(ExprevPE);
	optimalrPE = rvect(maxrevind);
	plot([optimalrPE optimalrPE],[0 maxExprevPE],'c--','LineWidth',3)
	grid on
	axis([200 415 0 310])
	h = legend('$E(R ; r, \widehat{\lambda}$)','$E(R ; r, \lambda_+)$','$E(R ; r, \lambda_-)$','$r^*$ (Optimal Reserve Price)');
	set(h,'Interpreter','latex','FontSize',12)
    
    save('RevCalcOrder7.mat','ExprevPE','hypIncExprevPE','hypDecExprevPE','optimalrPE','ngrid','rvect')
    betaB1PE = betaBPE(1:K+3);
    betaB2PE = betaBPE(K+4:end);
    
    [ir,outmat] = inforents(1000000,betaVPE,knotvecV,betaBPE,knotvecB,lambdaPE,delta1,delta2);
end

robustness = 0;
if robustness == 1
    display('')
    display('%%%Robustness Check: Simulating Filter Process under point estimates...')
    [N,NtildeSIA,Ntilde,highestlosersim,fptriggersim]...
                    = SimFilter(length(obsdata),betaVPE,knotvecV,betaBPE,knotvecB,lambdaPE,delta1,delta2,bstar);
    T=length(Ntilde);
    fptriggersim(fptriggersim==0) = nan;  %%%%We do this to conform to the input requirements of the KnitroLSqLambdaOptSimpleFilterGen file, which assumes that the third column contains either nan (if a second-price rule was triggered) or the high bid (if a first-price rule was triggered.
    
    display('')
    display('Estimating the Baseline case with direct observations of N...')
    [lambdaRC0]  = KnitroLSqLambdaOptSimpleFilterGenRC(N,lambdaPE,1,'fmincon');
    [betaVRC0,betaBRC0,epsilon0RC0,epsilon1RC0,epsilon2RC0,objfunvalMainRC0,exitflagMainRC0,outputMainRC0] =...
        VDistSplineEstim(bids,lambdaRC0,delta1,delta2,bstar,knotvecV,knotvecB,vdom,bdom,Fhat,L,solver,[],[]);
    
    display('')
    display('Estimating the comparison case with Ntilde but no SIA adjustment...')
    [lambdaRC]   = KnitroLSqLambdaOptSimpleFilterGen([Ntilde highestlosersim fptriggersim],ProbNgNtilde,lambdaPE,1,'fmincon',0);
    [betaVRC,betaBRC,epsilon0RC,epsilon1RC,epsilon2RC,objfunvalMainRC,exitflagMainRC,outputMainRC] =...
        VDistSplineEstim(bids,lambdaRC,delta1,delta2,bstar,knotvecV,knotvecB,vdom,bdom,Fhat,L,solver,[],[]);
        
    display('')
    display('Estimating the comparison case with SIA adjustment...')
    [lambdaRC2]   = KnitroLSqLambdaOptSimpleFilterGen([Ntilde highestlosersim fptriggersim],ProbNgNtilde,lambdaPE,1,'fmincon');
    [betaVRC2,betaBRC2,epsilon0RC2,epsilon1RC2,epsilon2RC2,objfunvalMainRC2,exitflagMainRC2,outputMainRC2] =...
        VDistSplineEstim(bids,lambdaRC2,delta1,delta2,bstar,knotvecV,knotvecB,vdom,bdom,Fhat,L,solver,[],[]);
    
        
%     vdom = linspace(knotvecV(1),knotvecV(end),1000)';
    [BasisV,~,DerivV] = BsplineBasis3(knotvecV,vdom,'off');
    figure(102)
%     subplot(2,1,1)
    hold on
    plot(vdom,BasisV*betaVRC0,'b','LineWidth',2)
    plot(vdom,BasisV*betaVRC,'r','LineWidth',2)
    plot(vdom,BasisV*betaVRC2,'g','LineWidth',2)
    plot(bdom,BasisV*betaVRC0,'--k','LineWidth',2)
    bdom = linspace(knotvecB(1),knotvecB(end),1000)';
    temp = BsplineEval3(knotvecB,betaBRC0,bdom);
    bdom = interp1(temp,bdom,vdom);
    legend('$\hat{F}_{V}^{baseline}$','$\hat{F}_{V}^{raw}$','$\hat{F}_{V}^{adjusted}$','$\hat{F}_{V}^{mis-specified}$','Location','SouthEast')
%     subplot(2,1,2)
%     hold on
%     plot(vdom,DerivV*betaVRC0,'b','LineWidth',2)
%     plot(vdom,DerivV*betaVRC,'r','LineWidth',2)
%     plot(vdom,DerivV*betaVRC2,'g','LineWidth',2)
%     legend('TRUE DGP','CHECK','ADJUSTMENT','Location','NorthEast')
    
end

matlabpool close

toc