function [pv,bid,dInv1,dInv2,dInv3,paybid] = EAauction(N,vbar,delta,dist,a,b,algorithm,r)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
%EAauction(N,vbar,delta,'dist',a,b,algorithm,r) calculates the equilibrium bid function 
%of an electronic auction for the case of independent private values 
%distributed on the interval [0,vbar], with N bidders and bid increment  
%delta.  See Hickman (2009) for the underlying theory.
%
%%%%%%%%%%%%%%%%%%%%%%%   INPUTS:   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%The mandatory inputs are N, vbar, delta and 'dist'. The other three are
%optional.  Although This program explicitly computes the equilibrium of a
%model where private values belong to a compact set, the unbounded case can
%be computed by choosing a large vbar where the difference between the CDF 
%F(vbar) and 1 is near machine precision.
%
%Dist, a and b determine the distribution of private values.  The six 
%options for 'dist' are 'Lognormal', 'Normal', 'Rayleigh', 'Uniform', 
%'Weibull', and 'Power' (note that 'dist' is a string argument).  Arguments
%a and b are distributional parameters.  
%
%For the lognormal case, a is the mean of the underlying normal and b is
%the standard deviation.  If a/b are left unspecified, then b is set
%equal to vbar/6 and a is chosen so that the mean is at vbar/2.  For the 
%normal case, a is the mean and b is the standard deviation.  If a
%is left unspecified, then it is set equal to vbar/2.  If b is left
%unspecified, it is set to vbar/6.  For the Rayleigh case, a is the parameter
%and b is ignored.  If a is left unspecified, then it is chosen so that the 
% mean is vbar/2.  For the Weibull case, a is the scale 
%parameter and b is the shape parameter.  When b=1, the Weibull becomes an 
%Exponential distribution with parameter a^(-1): f(x)=a^(-1)*exp(-x/a).  When a 
%and b are left unspecified, b is set equal to 1 and a is chosen so that 
%the mean is vbar/2.  For the uniform case, a and b are ignored and the
%parameters become 0 and vbar.  The power distribution is just a uniform CDF 
%raised to an exponent of a>=1.  If no value for a is specified, the
%default is 1.1.
%
%The final input parameter, algorithm, has two permissable values: 4 and 6.
%This determines the accuracy of the numerical approximation of the EA
%equilibrium .  A value of 4 means that a fourth-order Runge-Kutta
%algorithm will be used, and a value of 6 means a sixth-order
%RKA.  Either version uses a stepsize, h, chosen on line 148 below.  
%The 4th-order version has an O(h^4) error term and the 
%error on the 6th-order version is O(h^5).  On the other hand, run-time for 
%the latter is typically twice as long, so the default value of algorithm
%is 4.  
%
%The syntax is: 
%               [pv,bid]=EAauction(N,vbar,delta,'dist',a,b,algorithm) or 
%               [pv,bid]=EAauction(N,vbar,delta,'dist',a,b) or 
%               [pv,bid]=EAauction(N,vbar,delta,'dist',a) or
%               [pv,bid]=EAauction(N,vbar,delta,'dist'), depending on
%               whether the user wishes to specify distributional
%               parameters.
%
%%%%%%%%%%%%%%%%%%%%%%%   OUTPUTS:   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%The mandatory output arguments are pv, the domain grid (pv stands for 
%private values) and bid, the functional values at the grid points.  The
%optional output arguments are the first, second and third derivatives of
% the equilibrium inverse bid function, dInv1, dInv2 and dInv3, 
%respectively; and paybid, which gives the probability of paying one's bid 
%conditional on winning.  The syntax with all arguments is:
%   [pv,bid,dInv1,dInv2,dInv3,paybid] = EAauction(N,vbar,delta,'dist',a,b,algorithm)
%
%See also FPAUCTION 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%  DISTRIBUTIONAL SPECIFICATION:  %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin < 7
    algorithm = 4;
end

if nargin < 4
    error('At a minimum, you must specify N, vbar, delta and dist');
elseif nargout < 2
    error('Too few output arguments');
end

if nargin<8
    r=0;
end
%Retrieve the initial section of the function from FPauction.m
[pvtemp,bidtemp] = FPauctionR(N,vbar,r,dist,a,b,r+delta);

%In the following conditional loop, stepsize is chosen so that there will 
%be a minimum of 5 steps between increments of delta, or around 
%300-400 grid points overall (see else).  At the same time, it first checks 
%to make sure that the number of grid points are not too large, when delta 
%is very small.
if vbar/delta>300
    J=2;
elseif vbar/delta>600
    J=1;        
else
    stepsize = min([vbar/400, delta/5]);
	stepsize = stepsize/5;
    J = ceil(delta/stepsize);
end
J=J*10;
h = delta/J;
bid = [r:h:r+delta]';
pv  = spline(bidtemp,pvtemp,bid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Below we compute a vector dInv1 to store the slope of the inverse bid 
%function, for use later on in computing pvlag (see below).  We also compute
%the optional output vectors dInv2 and dInv3, which store the second and
%third derivatives of the equilibrium bid function.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
G1    = Gprime1(pv(1:end-1),N,0,vbar,dist,a,b); %This is the density of the highest opponent type
Al    = G(pv(1:end-1),N,0,vbar,dist,a,b);
Tom   = ( (pv(1:end-1)-bid(1:end-1)).*G1 ).^(-1);
dInv1 = Al.*Tom;  %Note from the above two lines that we don't compute the  
                  %slope of the inverse bid function slope for the last 
                  %element. This will happen in the first iteration of the 
                  %RK routine below.
dInv1(1) = dInv1(2) + (dInv1(2) - dInv1(3));
for i=find(isnan(dInv1)==1)  %This loop looks for places where the derivative is numerically unstable when computed by the analytic formula, and approximates it via finite differences.
      dInv1(i)=(pv(i)-pv(i+1))./(bid(i)-bid(i+1));
end

if nargout >=4  %%%% This will trigger computation of the 2nd derivative of the inverse bid function dInv2
    G2    = Gprime2(pv(1:end-1),N,0,vbar,dist,a,b); %This is the second derivative of the distribution of the highest opponent type
    G3    = Gprime3(pv(1:end-1),N,0,vbar,dist,a,b); %This is the third derivative of the distribution of the highest opponent type
    Dick  = G1.*dInv1;
    Harry = (dInv1-1).*G1 + (pv(1:end-1)-bid(1:end-1)).*G2.*dInv1;
    Eddy  = -(Tom.^2).*Harry;
    dInv2 = Dick.*Tom + Eddy.*Al;
	dInv2(1) = dInv2(2) + (dInv2(2) - dInv2(3));
end

if nargout >=5  %%%% This will trigger computation of the 3rd derivative of the inverse bid function dInv3
	dInv3 = ( G2.*dInv1.^2 + G1.*dInv2 ).*Tom + 2*Eddy.*(Dick - Tom.*Harry.*Al)...
		- (Tom.^2).*Al.*( dInv2.*G1 + 2*( dInv1-1 ).*G2.*dInv1 + (pv(1:end-1)-bid(1:end-1)).*( G3.*dInv1.^2 + G2.*dInv2 ) );
	dInv3(1) = dInv3(2) + (dInv3(2) - dInv3(3));
end
clear Tom Dick Harry Al Eddy G1 G2 G3


%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%  RUNGE-KUTTA:  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%The following is a Runge-Kutta algorithm for solving the differential 
%equation which defines equilibrium bidding in the EA auction.
%See equation (10) of Hickman 2009.
if algorithm == 4
    while pv(end) < vbar
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
        %%%%@#@#@#@#@#@#@#@#@#@#@# FOURTH-ORDER @#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@@#@#@#@#@#@#@#
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
            n   = length(bid);                                                  
            k1  = h*( G(pv(n),N,0,vbar,dist,a,b) - G(pv(n-J),N,0,vbar,dist,a,b) )/...     
                    (( pv(n) - bid(n) )*Gprime1(pv(n),N,0,vbar,dist,a,b));                 
                                                                                
            dInv1=[dInv1; k1/h];
            %In the next few lines, we reference back to a previously solved
            %segment of the function to compute pvlag.  However, the point
            %at which the function is evaluated lies between two grid
            %points, so we approximate it with a local cubic polynomial,
            %using previously computed information on levels and slopes.
            A = [1 bid(n-J)   bid(n-J)^2   bid(n-J)^3;...
                 1 bid(n-J+1) bid(n-J+1)^2 bid(n-J+1)^3;...
                 0 1          2*bid(n-J)   3*bid(n-J)^2;...
                 0 1          2*bid(n-J+1) 3*bid(n-J+1)^2];
            B = [pv(n-J); pv(n-J+1); dInv1(n-J); dInv1(n-J+1)];
            coef = A\B;
            arg = bid(n-J)+h/2;
            pvlag = coef(1) + coef(2)*arg + coef(3)*arg^2 + coef(4)*arg^3;
                                                                                
            k2  = h*( G(pv(n)+k1/2,N,0,vbar,dist,a,b) - G(pvlag,N,0,vbar,dist,a,b) )/...  
                    (( pv(n)+k1/2 - bid(n)-h/2 )*Gprime1(pv(n)+k1/2,N,0,vbar,dist,a,b));   
                                                                                
            k3  = h*( G(pv(n)+k2/2,N,0,vbar,dist,a,b) - G(pvlag,N,0,vbar,dist,a,b) )/...  
                    (( pv(n)+k2/2 - bid(n)-h/2 )*Gprime1(pv(n)+k2/2,N,0,vbar,dist,a,b));   
                                                                                
            k4  = h*( G(pv(n)+k3,N,0,vbar,dist,a,b) - G(pv(n-J+1),N,0,vbar,dist,a,b) )/...
                    (( pv(n)+ k3 - bid(n) - h )*Gprime1(pv(n)+k3,N,0,vbar,dist,a,b));      
                                                                                
            pvnew  = pv(n) + k1/6 + k2/3 + k3/3 + k4/6;                         
            bidnew = bid(n) + h;   
            if bidnew > pvnew
                bidnew = pvnew-10^(-5);
            end
                                                                                
            pv  = [pv; pvnew];                                                  
            bid = [bid; bidnew];                                                
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
        %%%%@#@#@#@#@#@#@#@#@# END FOURTH-ORDER @#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#  
    end
elseif algorithm == 6    
    while pv(end) < vbar
        x = pv(end);
        y = bid(end);
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
        %%%%@#@#@#@#@#@#@#@#@#@#@# SIXTH-ORDER #@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
        %The following is a sixth-order Runge-Kutta algorithm for solving the
        %differential equation.    
            k1 = h*( x - y )...
                   *Gprime1(x,N,0,vbar,dist,a,b)/...
                   ( G(x,N,0,vbar,dist,a,b) -...
                   G(spline(bid,pv,y-delta),N,0,vbar,dist,a,b) );
               
           dInv1=[dInv1; k1/h];

            k2 = h*( x + h -...
                   (y+k1) )...
                   *Gprime1(x+h,N,0,vbar,dist,a,b)/...
                   ( G(x+h,N,0,vbar,dist,a,b) -...
                   G(spline(bid,pv,(y+k1)-delta),N,0,vbar,dist,a,b) );

            k3 = h*( x + h/2 -...
                   ( y+(3*k1 + k2)/8 ) )...
                   *Gprime1(x+h/2,N,0,vbar,dist,a,b)/...
                   ( G(x+h/2,N,0,vbar,dist,a,b) -...
                   G(spline(bid,pv,( y+(3*k1 + k2)/8 )-delta),N,0,vbar,dist,a,b) );

            k4 = h*( x + 2*h/3 -...
                   ( y + (8*k1 + 2*k2 + 8*k3)/27 ) )...
                   *Gprime1(x+2*h/3,N,0,vbar,dist,a,b)/...
                   ( G(x+2*h/3,N,0,vbar,dist,a,b) -...
                   G(spline(bid,pv,( y + (8*k1 + 2*k2 + 8*k3)/27 )-delta),N,0,vbar,dist,a,b) );

            k5 = h*( x + (7-21^.5)*h/14 -...
                   ( y + (3*(3*21^.5-7)*k1 - 8*(7-21^.5)*k2 + 48*(7-21^.5)*k3 - 3*(21-21^.5)*k4)/392 ) )...
                   *Gprime1(x+(7-21^.5)*h/14,N,0,vbar,dist,a,b)/...
                   ( G(x+(7-21^.5)*h/14,N,0,vbar,dist,a,b) -...
                   G(spline(bid,pv,( y + (3*(3*21^.5-7)*k1 - 8*(7-21^.5)*k2 + 48*(7-21^.5)*k3 - 3*(21-21^.5)*k4)/392 )-delta),N,0,vbar,dist,a,b) );

            k6 = h*( x + (7+21^.5)*h/14 -...
                   ( y + (-5*(231+51*21^.5)*k1 - 40*(7+21^.5)*k2 - 320*(21^.5)*k3 + 3*(21+121*21^.5)*k4 + 392*(6+21^.5)*k5)/1960 ) )...
                   *Gprime1(x+(7+21^.5)*h/14,N,0,vbar,dist,a,b)/...
                   ( G(x+(7+21^.5)*h/14,N,0,vbar,dist,a,b) -...
                   G(spline(bid,pv,( y + (-5*(231+51*21^.5)*k1 - 40*(7+21^.5)*k2 - 320*(21^.5)*k3 + 3*(21+121*21^.5)*k4 + 392*(6+21^.5)*k5)/1960 )-delta),N,0,vbar,dist,a,b) );

            k7 = h*( x + h -...
                   ( y + (15*(22+7*21^.5)*k1 + 120*k2 + 40*(7*21^.5-5)*k3 - 63*(3*21^.5-2)*k4 - 14*(49+9*21^.5)*k5 + 70*(7-21^.5)*k6)/180 ) )...
                   *Gprime1(x+h,N,0,vbar,dist,a,b)/...
                   ( G(x+h,N,0,vbar,dist,a,b) -...
                   G(spline(bid,pv,( y + (15*(22+7*21^.5)*k1 + 120*k2 + 40*(7*21^.5-5)*k3 - 63*(3*21^.5-2)*k4 - 14*(49+9*21^.5)*k5 + 70*(7-21^.5)*k6)/180 )-delta),N,0,vbar,dist,a,b) );

            pvnew  = x + h;
            bidnew = y + ( 9*k1 + 64*k3 + 49*k5 + 49*k6 + 9*k7 )/180;

            pv  = [pv; pvnew];
            bid = [bid; bidnew];
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
        %%%%@#@#@#@#@#@#@#@#@# END SIXTH-ORDER #@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@
        %%%%#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#@#
    end
else
    display('Input variable algorithm must be either 4 or 6, depending on')
    display('whether a Runge-Kutta algorithm of 4th or 6th order is desired.')
    error()
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%  FINAL FUNCTIONAL VALUE:  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(pv(end)>vbar) || isnan(pv(end)) || isinf(pv(end))
    pv(end)=[];
    bid(end)=[];
%This final segment ensures that the final domain point in the vector pv is
%exactly vbar.
    h = vbar - pv(end); 
    x = pv(end);
    y = bid(end);
    k1  = h*( x - y )*Gprime1(x,N,0,vbar,dist,a,b)/( G(x,N,0,vbar,dist,a,b)-G(spline(bid,pv,y-delta),N,0,vbar,dist,a,b) );
    k2  = h*( x+h/2 - (y+k1/2) )*Gprime1(x+h/2,N,0,vbar,dist,a,b)/( G(x+h/2,N,0,vbar,dist,a,b)-G(spline(bid,pv,(y+k1/2)-delta),N,0,vbar,dist,a,b) );
    k3  = h*( x+h/2 - (y+k2/2) )*Gprime1(x+h/2,N,0,vbar,dist,a,b)/( G(x+h/2,N,0,vbar,dist,a,b)-G(spline(bid,pv,(y+k2/2)-delta),N,0,vbar,dist,a,b) );
    k4  = h*( x+h - (y+k3) )*Gprime1(x+h,N,0,vbar,dist,a,b)/( G(x+h,N,0,vbar,dist,a,b)-G(spline(bid,pv,(y+k3)-delta),N,0,vbar,dist,a,b) );
    pv(end)  = x + h;   %This forces the final domain point to be exactly vbar.
    bid(end) = y + k1/6 + k2/3 + k3/3 + k4/6;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%  OTHER OUTPUT ARGUMENTS:  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Below we compute a vector dInv1 to store the slope of the inverse bid 
%function, for use later on in computing pvlag (see below).  We also compute
%the optional output vectors dInv2 and dInv3, which store the second and
%third derivatives of the equilibrium bid function.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargout >=4  %%%% This will trigger computation of the 2nd derivative of the inverse bid function dInv2
    start = length(dInv2)+1;
    fin   = length(pv);
    %%% Recall that dInv2 and dInv3 have already been computed for the
    %%% initial segment where the first-price rule is certain.  Hereafter,
    %%% variables in all caps represent the objects restricted to bids
    %%% weakly exceeding Delta.
    PV    = pv(start:fin);
    BID   = bid(start:fin);
    DINV1 = dInv1(start:fin);
    DINV1LAG = dInv1([start:fin]-J);
    G1    = Gprime1(PV,N,0,vbar,dist,a,b); %This is the density of the highest opponent type
    G2    = Gprime2(PV,N,0,vbar,dist,a,b); %This is the second derivative of the distribution of the highest opponent type
    G3    = Gprime3(PV,N,0,vbar,dist,a,b); %This is the third derivative of the distribution of the highest opponent type
    GLAG  = G(pv([start:fin]-J),N,0,vbar,dist,a,b); %Recall that J was engineered so that bid(n)-bid(n-J)=Delta.
    G1LAG = Gprime1(pv([start:fin]-J),N,0,vbar,dist,a,b);
    G2LAG = Gprime2(pv([start:fin]-J),N,0,vbar,dist,a,b);
    Larry = G(PV,N,0,vbar,dist,a,b) - GLAG;
    Mo    = ( (PV-BID).*G1 ).^(-1);
    Chuck = G1.*DINV1 - G1LAG.*DINV1LAG;
    Bob   = (DINV1-1).*G1 + (PV-BID).*G2.*DINV1;
    Curly = -(Mo.^2).*Bob;
    DINV2 = Chuck.*Mo + Curly.*Larry;
    dInv2 = [dInv2; DINV2];
	%dInv2(1)=(dInv1(2)-dInv1(3))/(bid(2)-bid(3));
	for i=find(isnan(dInv2)==1)  %This loop looks for places where the derivative is numerically unstable when computed by the analytic formula, and approximates it via finite differences.
		dInv2(i)=(dInv1(i)-dInv1(i+1))/(bid(i)-bid(i+1));
	end
end


if nargout >=5  %%%% This will trigger computation of the 3rd derivative of the inverse bid function dInv3
    DINV3 = ( G2.*DINV1.^2 + G1.*DINV2 - G2LAG.*DINV1LAG.^2 - G1LAG.*dInv2([start:fin]-J)).*Mo...
            + 2*Curly.*(Chuck - Mo.*Bob.*Larry)...
            - (Mo.^2).*Larry.*( DINV2.*G1 + 2*( DINV1-1 ).*G2.*DINV1 + (PV-BID).*( G3.*DINV1.^2 + G2.*DINV2 ) );
    dInv3 = [dInv3; DINV3];
	%dInv3(1)=(dInv2(2)-dInv2(3))/(bid(2)-bid(3));
	for i=find(isnan(dInv3)==1)  %This loop looks for places where the derivative is numerically unstable when computed by the analytic formula, and approximates it via finite differences.
		dInv3(i)=(dInv2(i)-dInv2(i+1))./(bid(i)-bid(i+1));
	end
end

% clear Mo Chuck Bob Larry Curly G1 G2 G3


paybid = 0;
if nargout > 5
    paybid=ones(length(find(bid<=delta)),1); 
    temp=pv(find(bid>delta)); 
    paybid = [paybid; (G(temp,N,0,vbar,dist,a,b)-G(spline(bid,pv,bid(find(bid>delta))-delta),N,0,vbar,dist,a,b))./G(temp,N,0,vbar,dist,a,b)];
end


% hold on;
% plot(pv,bid,pv,pv)
% toc