%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% see how bid increment affets frequency of FPA rule coming into play

clear all
clc
tic

% distribution of interest
vlow = 0;
vhigh = 1;
dist = 'rayleigh';
param1 = 0.3;
param2 = 4;
pvect = [param1 param2];
Nvect = [3 5 10];
deltavect = [.01 .02 .05];

% open file to write some output to
fid = fopen('FPAfreq.out','a');
if strcmpi(dist,'weibull')
	fprintf(fid,'\n%s(%.1f,%.1f)\n',dist,param1,param2);
else
	fprintf(fid,'\n%s(%.1f)\n',dist,param1);
end

for n=1:length(Nvect)
N = Nvect(n);

% number of auctions to consider in computing frequency
T = 1000000;
rand('state',81981);
u = rand(N*T,1);
v = inversecdf(u,dist,param1,param2,vlow,vhigh);

for i=1:length(deltavect)
delta = deltavect(i);

% solve for the inverse-bid function given the distribution
rkorder = 6;
[vals,bids] = EAauction(N,vhigh,delta,dist,param1,param2,rkorder);
if sum(vals - bids < 0) > 0
	warning('Bid function is not rational: some bids exceed valuations\n')
end

% use bid function to get bids that go with valuations
b = spline(vals,bids,v);
if sum(v - b < 0) > 0
	warning('Bid function is not rational: some bids exceed valuations\n')
end

% reshape so all bids from given auction are in row
bids = reshape(b,T,N);

% sort so high bid (winner) is in first column
bids = sort(bids','descend')';

% determine what pricing rule led to winner
fparule = bids(:,1) <= bids(:,2) + delta;
sparule = bids(:,1) > bids(:,2) + delta;
nfparule = sum(fparule);
nsparule = sum(sparule);
if nfparule + nsparule ~= T
	warning('Some auction(s) not assigned a pricing rule\n')
else
	if strcmpi(dist,'weibull')
		fprintf('\n%s(%.1f,%.1f)\n',dist,param1,param2);
	else
		fprintf('\n%s(%.1f)\n',dist,param1);
	end
	fprintf('Bid increment delta is: %.2f with %d bidders\n',delta,N);
	fprintf('FPA rule occurred %d times: %.2f percent of the time\n', ...
		nfparule,nfparule/T*100);
	fprintf('SPA rule occurred %d times: %.2f percent of the time\n', ...
		nsparule,nsparule/T*100);
	if N < 10
		fprintf(fid,'N=%d\t\t\\Delta=$%.2f$\t$%.2f\\%%$\n',N,delta, ...
			nfparule/(nfparule + nsparule)*100);
	else
		fprintf(fid,'N=%d\t\\Delta=$%.2f$\t$%.2f\\%%$\n',N,delta, ...
			nfparule/(nfparule + nsparule)*100);
	end
end
end
end
fclose(fid);
toc