function [ir,outmat] = inforents(T,betaV,knotvecV,betaB,knotvecB,lambdahat,delta1,delta2)
%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% compute ebay information rents

% evaluate parent valuation distribution at fine grid of points, which will
% be used with interpolation to map random uniform draws into valuations
ngrid = 1000000;
vgrid = linspace(knotvecV(1),knotvecV(end),ngrid)';
[basisV] = BsplineBasis3(knotvecV,vgrid);
FVgrid = basisV*betaV;

% draw random uniforms, assigning N bidders to auctions using the weights
% prescribed by lambda
maxN = 100;  
N = 1:1:maxN;
probN = poissgenpdf(N,lambdahat);
normconst = 1/(1 - poissgencdf(1,lambdahat));
weight = probN.*normconst;
numN = round(weight*T);
fprintf('%d auctions simulated for information rents\n',sum(numN(2:end)))

% set random number generator seed and generator matrix of random uniforms
%rng(2016);
rng(5*100000,'twister');
% could finesse this a bit, just making sure about T simulations are run
nrows = max(T,sum(numN(2:end)));
U = rand(nrows,maxN);

% map highest two uniform draws into valuations for given realization of N
top2u = [];
Nvect = [];
for i=2:maxN
	tempu = [];
	ind = size(top2u,1);
	tempu = U(ind+1:ind+numN(i),1:N(i));
	sortu = sort(tempu');
	u1 = sortu(end,:);
	u2 = sortu(end-1,:);
	top2u = [top2u; u1' u2'];
	Nvect = [Nvect; N(i)*ones(numN(i),1)];
end
if size(top2u,1) ~= sum(numN(2:end))
	fprintf('Dimension mismatch in assigning highest valuations\n')
end

% map two highest uniform draws into two highest valuations
v1 = interp1(FVgrid,vgrid,top2u(:,1),'pchip');
v2 = interp1(FVgrid,vgrid,top2u(:,2),'pchip');

% evaluate inverse bid function at fine grid of points, which will be used
% with interpolation to map valuations into bids
bgrid = linspace(knotvecB(1),knotvecB(end),ngrid)';
[basisB] = BsplineBasis3(knotvecB,bgrid);
ibfvgrid = basisB*betaB;

% map two highest valuations into two highest bids
b1 = interp1(ibfvgrid,bgrid,v1,'pchip');
b2 = interp1(ibfvgrid,bgrid,v2,'pchip');

%GB = BsplineEval3(knotvecV,betaV,bgrid);
FV = BsplineEval3(knotvecV,betaV,ibfvgrid);
GB = FV;
b1spa = interp1(GB,bgrid,top2u(:,1),'pchip');
b2spa = interp1(GB,bgrid,top2u(:,2),'pchip');


% check if first-price rule holds
p = min(b2 + delta1,b1);
p(b2 >= 250) = min(b2(b2 >= 250) + delta2,b1(b2 >= 250));

ir = v1 - p;

irspa = b1spa - b2spa;

outmat = [v1 b1 v2 b2 p ir Nvect irspa];

