%  Brent R. Hickman, Timothy P. Hubbard, and Harry J. Paarsch
% hickmanbr@uchicago.edu, timothy.hubbard@colby.edu, hjpaarsch@gmail.com
%
% please cite our Quantitative Economics paper "Identification and
% Estimation of a Bidding Model for Electronic Auctions" if this code is
% helpful to you.
%
% compare estimation of EA and SP auctions to see importance of bid
% increments

clear all
clc
% parameters for figure properties
lwidth = 2;
fsize = 14;
set(0,'defaulttextinterpreter','latex','Defaulttextfontsize',fsize);

% set seed
rand('state',111)

% number of simulations, auctions per simulation, and bidders at auction
T = 1000;
N = 5;

% distribution of interest
vlow = 0;
vhigh = 1;
% if exponential think of param1 = lambda (probably want to be greater than
% 1...although mean is different given truncation); if rayleigh probably
% want mean to be less than sqrt(pi)/2 so that mean is less than one.
% weibull is an exponential if param2 = 1 and is a rayleigh if param2 = 2.
% in weibull being rayleigh case, weibull param1 = sqrt(2*rparam1^2) will
% make them equivalent, where rparam1 is rayleigh param1
dist = 'weibull';
param1 = 0.5;
param2 = 4;
pvect = param1;
if strcmpi(dist,'weibull')
	pvect = [pvect param2];
end
delta = 0.02;

% solve for the inverse-bid function given the distribution
rkorder = 4;
[vals,bids] = EAauction(N,vhigh,delta,dist,param1,param2,rkorder);
if sum(vals - bids < 0) > 0
	warning('Bid function is not rational: some bids exceed valuations\n')
	return
end

% true optimal reserve price
rhoguess = (vhigh + vlow)/2;
rho_true = fzero('optreserve',rhoguess,[],dist,vhigh,pvect);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 1: get randomly drawn valuations and bids given distribution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% generate valuations from the distribution of interest via inverse cdf
u = rand(N*T,1);
v = inversecdf(u,dist,param1,param2,vlow,vhigh);

% this is the best we could hope to achieve given randomly generated data
options = optimset('GradObj','on','Hessian','on','Display','off', ...
	'TolFun',1e-12,'TolX',1e-16,'MaxIter',1000,'MaxFunEvals',1000, ...
	'DerivativeCheck','off','LargeScale','on');
[thetahat,L,flag,output,grad,hess] = fminunc('rawllhood',pvect,options, ...
	v,dist,vhigh);


% get EA inverse-bid function evaluated at the randomly-drawn valuations so
% that we have randomly-drawn bids
b = spline(vals,bids,v);
if sum(v - b < 0) > 0
	warning('Bid function is not rational: some bids exceed valuations\n')
	return
end
if (mean(v) > vhigh/2) && strcmpi(dist,'exponential')
	warning('MLE estimator will not exist for this data\n')
	return
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 2: do nonparametric estimations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
kernel = 'triweight';
% b0 is needed for h0 bandwidth---see KZ (2008) bottom of page 501 and
% the K0 term in (4.9) of ZK (1998) which is K0 = 6 + 18t + 12t^2,
% t\in[-1,0]; this will compute b0 for any kernel choice provided by
% user
num_t1 = quad(@evalkspdf_num,-1,1,1e-12,[],kernel);
num_t2 = 0 - (-144/5 + 432/4 - 468/3 + 216/2 - 36);
denom_t1 = 0 - (-2 + 18/4 - 12/5);
denom_t2 = quad(@evalkspdf_denom,-1,1,1e-12,[],kernel);
b0 = ((num_t1^2*num_t2)/(denom_t1^2*denom_t2))^(1/5);


vplot = linspace(0,1,1000);

% in spa, do gpv on bids directly as b = v
v_spa = b;
[fbc_spa,h_spa] = kspdf_bc(v_spa,kernel,vplot,b0);


% in ea, do gpv on correctly-specified pseudo-values
[gb,hg_ea] = kspdf_bc(b,kernel,b,b0);
Gb = kscdf_new(b,'edf',b);
Gb_delta = kscdf_new(b,'edf',b-delta);
v_ea = b' + (Gb.^(N - 1) - Gb_delta.^(N - 1))./((N - 1).*gb.*Gb.^(N - 2));
v_ea = v_ea';
[fbc_ea,hf_ea] = kspdf_bc(v_ea,kernel,vplot,b0);

% best parametric estimate we could hope for given sample of valuations
a = thetahat(1);
if strcmpi(dist,'weibull')
	b = thetahat(2);
	F = 1 - exp(-(vplot./a).^b);
	f = (b/a).*(vplot./a).^(b - 1).*exp(-(vplot./a).^b);
	Ftr = 1 - exp(-(vhigh/a)^b);
	Fv = F./Ftr;
	fv = f./Ftr;
elseif strcmpi(dist,'rayleigh')
	F = 1 - exp(-(vplot.^2)./(2*a^2));
	f = vplot./a^2.*exp(-(vplot.^2)./(2*a^2));
	Ftr = 1 - exp(-(vhigh^2)./(2*a^2));
	Fv = F./Ftr;
	fv = f./Ftr;
end
[fbc_v,h_v] = kspdf_bc(v,kernel,vplot,b0);


figure
set(gcf,'DefaultLineLineWidth',lwidth)
set(gca,'FontSize',fsize)
hold all
box on
plot(vplot,fbc_v,vplot,fbc_ea,'-.',vplot,fbc_spa,'--')
xlabel('$\mathbf{v}$')
xlim([vlow vhigh])
L = legend('$\mathbf{\hat{f}(v)}$', ...
	'$\mathbf{\hat{f}_{EA}(v)}$', ...
	'$\mathbf{\hat{f}_{SP}(v)}$');
set(L,'Interpreter','latex','Location','NorthEast')
print('-depsc','npcompare.eps')